import java.text.DecimalFormat;

public final class ProcessedLogStats {
  public long numBLPPNodes=0;
  public long numKBLPPNodes=0;
  public long numKBLPPTrees=0;    
  public long processedEvents=0;
  public long numBLPPGetEvents=0;
  public long numBLPPPutEvents=0;
  public long numKBLPPGetEvents=0;
  public long numKBLPPPutEvents=0;
  public long sumOfBLPPcounters=0;
  public long sumOfKBLPPcounters=0;
  public long numKSFinternalNodes=0;
  public long numKSFchildren=0;
  public long sumKSFchildLookups=0;
  public long sumKSFchildCounters=0;
  public double avgKSFdegree=0;
  public double avgKSFperChildLookup=0;
  
  public long numIPFnodes=0;
  public long numIPFroots=0;
  public long sumIPFrootCounters=0;
  public long numIPFleaves=0;
  public long sumIPFleafDepths=0;
  public long numIPFinternalNodes=0;
  public long numIPFchildren=0;
  public double avgIPFleafDepth=0;
  public double avgIPFdegree=0;  
  
  public long eventsWithinK[];
  public long valuesForK[];
    
  public ProcessedLogStats() {
    // possible values for k: 1 2 3 5 7 10 15
    eventsWithinK=new long[7];
    valuesForK=new long[7];
    valuesForK[0]=1;
    valuesForK[1]=2;
    valuesForK[2]=3;
    valuesForK[3]=5;
    valuesForK[4]=7;
    valuesForK[5]=10;
    valuesForK[6]=15;
  }
  
  public final String toString() {            
    /* Fields inside the string:
     * processedEvents acyclicKSFnodes cycleKSFnodes totalKSFNodes
     * acyclicHashes cycleHashes totalHashes hashFractionCyclesOnly
     * hashFractionGlobal sumBLPPcounters sumKBLPPcounters numIPFnodes
     * numIPFtrees sumIPFrootCounters numIPFleaves avgIPFleafDepth
     * numIPFinternal avgIPFdegree numKSFinternal avgKSFdegree */
    long blppHashes=numBLPPGetEvents+numBLPPPutEvents;
    long kblppHashes=numKBLPPGetEvents+numKBLPPPutEvents;
    double fracCycles=computeFractionOfAccessesForCyclesWrtBLPP();
    double fracTot=computeFractionOfAccessesWrtBLPP();
    
    DecimalFormat myFormat = new DecimalFormat("#0.######");
    String fracCycleStr=myFormat.format(fracCycles);
    String fracTotStr=myFormat.format(fracTot);
    String avgLeafDepth=myFormat.format(avgIPFleafDepth);
    String avgIPFdegreeString=myFormat.format(avgIPFdegree);
    String avgKSFdegreeString=myFormat.format(avgKSFdegree);
    String avgKSFlookups=myFormat.format(avgKSFperChildLookup);
          
    String ret = processedEvents+" "+numBLPPNodes+" "+numKBLPPNodes+" "+(numBLPPNodes+numKBLPPNodes)+" "
                 +blppHashes+" "+kblppHashes+" "+(blppHashes+kblppHashes)+" "+fracCycleStr+" "
                 +fracTotStr+" "+sumOfBLPPcounters+" "+sumOfKBLPPcounters+" "+numIPFnodes+" "
                 +numIPFroots+" "+sumIPFrootCounters+" "+numIPFleaves+" "+avgLeafDepth+" "
                 +numIPFinternalNodes+" "+avgIPFdegreeString+" "+numKSFinternalNodes+" "
                 +avgKSFdegreeString+" "+avgKSFlookups;

    // replace "," with "." as decimal separator
    return ret.replace(",", ".");               
  }
  
  public final double computeFractionOfAccessesForCyclesWrtBLPP() {
    double fraction=0;
    if (sumOfKBLPPcounters > 0)
      fraction = ((double)(numKBLPPGetEvents+numKBLPPPutEvents))/(sumOfKBLPPcounters+numKBLPPNodes);
    return fraction;    
  }
  
  public final double computeFractionOfAccessesWrtBLPP() {
    double fraction=0;
    if (sumOfBLPPcounters > 0 || sumOfKBLPPcounters > 0)
      fraction = ((double)(numKBLPPGetEvents+numKBLPPPutEvents+numBLPPGetEvents+numBLPPPutEvents))
                 / (sumOfKBLPPcounters+numKBLPPNodes+numBLPPGetEvents+numBLPPPutEvents);
    return fraction;      
  }
  
  public final static void updateProgramStats(ProcessedLogStats global, ProcessedLogStats current) {
    global.numBLPPNodes += current.numBLPPNodes;
    global.numKBLPPNodes += current.numKBLPPNodes;    
    global.numKBLPPTrees += current.numKBLPPTrees;    
    global.processedEvents += current.processedEvents;
    global.numBLPPGetEvents += current.numBLPPGetEvents;
    global.numBLPPPutEvents += current.numBLPPPutEvents;
    global.numKBLPPGetEvents += current.numKBLPPGetEvents;
    global.numKBLPPPutEvents += current.numKBLPPPutEvents;
    global.sumOfBLPPcounters += current.sumOfBLPPcounters;
    global.sumOfKBLPPcounters += current.sumOfKBLPPcounters;
    global.numKSFchildren += current.numKSFchildren;
    global.numKSFinternalNodes += current.numKSFinternalNodes;
    global.sumKSFchildLookups += current.sumKSFchildLookups;
    global.sumKSFchildCounters += current.sumKSFchildCounters;
    
    global.numIPFnodes += current.numIPFnodes;
    global.numIPFroots += current.numIPFroots;
    global.sumIPFrootCounters += current.sumIPFrootCounters;
    global.numIPFleaves += current.numIPFleaves;
    global.sumIPFleafDepths += current.sumIPFleafDepths;    
    global.numIPFinternalNodes += current.numIPFinternalNodes;
    global.numIPFchildren += current.numIPFchildren;
    
    if (global.numIPFleaves>0)
      global.avgIPFleafDepth = ((double)global.sumIPFleafDepths)/global.numIPFleaves;
    if (global.numIPFinternalNodes>0)
      global.avgIPFdegree=((double)global.numIPFchildren)/global.numIPFinternalNodes;  
    if (global.numKSFinternalNodes>0)
      global.avgKSFdegree=((double)global.numKSFchildren)/global.numKSFinternalNodes;
    if (global.numKSFchildren>0)
      global.avgKSFperChildLookup=((double)global.sumKSFchildLookups)/global.sumKSFchildCounters;
        
    for (int i=0; i<global.valuesForK.length; i++)
      global.eventsWithinK[i]+=current.eventsWithinK[i];
  }
  
} 
